CLASS Acme.DAOrder
    INHERITS Acme.DAOrderSuper
    IMPLEMENTS Acme.IList,
               Acme.IDataAccess :

    /**************** INTERFACE IDataAccess METHODS SECTION *********************/
    
     METHOD PUBLIC LONGCHAR  selectRecords (INPUT cWhere AS CHARACTER):
        DEFINE VARIABLE hdsOrder     AS HANDLE     NO-UNDO.
        DEFINE VARIABLE hEvents      AS HANDLE     NO-UNDO.
        DEFINE VARIABLE iBuff        AS INTEGER    NO-UNDO.
        DEFINE VARIABLE hBuff        AS HANDLE     NO-UNDO.
        DEFINE VARIABLE lcXMLDataSet AS LONGCHAR   NO-UNDO.
        
        DEFINE QUERY qOrder FOR Sports2000.Order, Sports2000.Customer, Sports2000.SalesRep.
        
        DEFINE DATA-SOURCE srcOrder FOR QUERY qOrder 
            Order KEYS (OrderNum), Customer KEYS (CustNum), SalesRep KEYS (SalesRep).
        DEFINE DATA-SOURCE srcOline FOR OrderLine.
                
        hdsOrder = DATASET dsOrder:HANDLE.
        /* Use NEW SET-CALLBACK Method for THIS-OBJECT instead of persistent proc */
        SetCallBacks(INPUT hdsOrder).
        
        /* Prepare the query */
        QUERY qOrder:QUERY-PREPARE("FOR EACH Order WHERE order.ordernum = " + (cWhere) + ", FIRST Customer OF Order, FIRST SalesRep OUTER-JOIN OF Order").
        
        /* Attach datasources before fill */
        BUFFER ttOrder:ATTACH-DATA-SOURCE(DATA-SOURCE srcOrder:HANDLE,
                                               "Customer.Name,CustName").
        BUFFER ttOline:ATTACH-DATA-SOURCE(DATA-SOURCE srcOline:HANDLE).
                   
        hDSOrder:FILL().
        
        /*Detach datasources after fill */
        DO iBuff = 1 TO DATASET dsOrder:NUM-BUFFERS:
            DATASET dsOrder:GET-BUFFER-HANDLE(iBuff):DETACH-DATA-SOURCE().
        END.
            
        /* Prepare return parameter */                 
        convertToXML(OUTPUT lcXMLDataSet).
        RETURN lcXMLDataSet.
        
    END METHOD. /* end method selectRecords */

    /**************** INTERFACE IList METHODS SECTION *********************/
    
    METHOD PUBLIC VOID nextRecord  ( ):
         /* will navigage to next DataObject */
    END METHOD. /* end method nextRecord */

    METHOD PUBLIC VOID prevRecord  ( ):
        /* will navigage to prev DataObject */
    END METHOD. /* end method prevRecord */

    METHOD PUBLIC VOID firstRecord ( ):
        /* will navigage to first DataObject */
    END METHOD. /* end method firstRecord */

    METHOD PUBLIC VOID lastRecord  ( ):
        /* will navigage to last DataObject */
    END METHOD. /* end method lastRecord */
        
    /****************  METHODS FOR DAORDER CLASS SECTION *********************/

    CONSTRUCTOR PUBLIC DAOrder (INPUT iOrderNum AS INTEGER):
    
        FIND Sports2000.Order WHERE (Sports2000.Order.OrderNum EQ iOrderNum) NO-ERROR.
        IF NOT AVAILABLE (Sports2000.Order) THEN
            DELETE OBJECT THIS-OBJECT.
            
    END CONSTRUCTOR. /* end constructor DAOrder */
            
    METHOD PRIVATE VOID setCallBacks ( phDataSet AS Handle):
    
        DEFINE VARIABLE iBuff     AS INTEGER    NO-UNDO.
        DEFINE VARIABLE hBuff     AS HANDLE     NO-UNDO.
        
        phDataSet:SET-CALLBACK
                 ("AFTER-FILL", "postDataSetFill", THIS-OBJECT).
                 
        DO iBuff = 1 TO phDataSet:NUM-BUFFERS:
                 hBuff = phDataSet:GET-BUFFER-HANDLE(iBuff).
            IF hBuff:NAME = "ttOline" THEN
                hBuff:SET-CALLBACK ("AFTER-FILL", "postOlineFill", THIS-OBJECT).
        END. /* Do iBuff */
        
    END METHOD. /* end method setCallBacks */
    
    
  
    METHOD PUBLIC VOID postOlineFill (INPUT DATASET FOR dsOrder):
    
         DEFINE VARIABLE dTotal AS DECIMAL    NO-UNDO.
     
         /* All orderlines collected so total the order and store in ttorder */
         FOR EACH ttOline WHERE ttOline.OrderNum = 
             ttOrder.OrderNum:
               dTotal = dTotal + ttOline.ExtendedPrice.
         END.
         ttOrder.OrderTotal = dTotal.
         
     END METHOD. /* end method postRecordFill */
     
     
     
     METHOD PUBLIC VOID postDataSetFill (INPUT DATASET FOR dsOrder):
     
        FIND FIRST ttOrder NO-LOCK NO-ERROR.
        IF AVAILABLE ttOrder THEN
            IF ttOrder.RepName = "" THEN 
                ASSIGN ttOrder.RepName = "No Assigned Rep!".
                
     END METHOD. /* end method postDataSetFill */

    METHOD PRIVATE VOID convertToXML(OUTPUT lcXMLDATASET AS LONGCHAR):
        DATASET dsOrder:WRITE-XML ("LONGCHAR", /* target-type */
                   lcXMLDATASET, /* longchar */
                   TRUE, /* formatted */
                   ?, /* encoding */
                   ?, /* schema location */
                   ?, /* write schema */
                   ?). /* minschema */
    
    END. /* end of ConvertToXML */
  
    /* This method is a 'stub' for the real business logic to initiate a cross
    ** charge between departments.
    */
    METHOD PUBLIC VOID setCrossCharge (INPUT iCrossChargeNum AS INTEGER):
        MESSAGE "To Do: implement setCrossCharge for Dept #" iCrossChargeNum
                VIEW-AS ALERT-BOX INFO BUTTONS OK.
    END. /* end of setCrossCharge */

    /* This method is a 'stub' for the real business logic to initiate an invoice */
    METHOD PUBLIC VOID sendInvoiceInfo (INPUT iInvoiceNum AS INTEGER):
        MESSAGE "To Do: sendInvoiceInfo for Invoice #" iInvoiceNum
                VIEW-AS ALERT-BOX INFO BUTTONS OK.
    END. /* end of sendInvoiceInfo */

END CLASS. /* end class Acme.DAOrder */
